// Copyright (C) 2004-2022, Stefan Kanthak <stefan.kanthak@nexgo.de>

// https://msdn.microsoft.com/en-us/library/4141z1cx.aspx

#pragma once

#ifndef __VERSION_H__
#define __VERSION_H__

#if ANYSIZE_ARRAY != 1
#error ANYSIZE_ARRAY must be defined as 1!
#endif

#define VS_BINARY	0U
#define VS_TEXT		1U

#pragma pack(show)
#pragma pack(push, 4)

// https://msdn.microsoft.com/en-us/library/ms647001.aspx

typedef	struct	_VS_VERSIONINFO
{
	WORD	wSize;			// size of 'VERSION' resource
	WORD	wCount;			// = sizeof(VS_FIXEDFILEINFO)
					//   (number of bytes in binary value)
	WORD	wType;			// = VS_BINARY
	WCHAR	szKey[16];		// = L"VS_VERSION_INFO"
	WORD	wPadding;		// = 0 (alignment to DWORD)
#if 0
	DWORD	dwValue[13];
#else
	// https://msdn.microsoft.com/en-us/library/ms646997.aspx

	VS_FIXEDFILEINFO	vsFFI;
#endif
} VS_VERSIONINFO;

// https://msdn.microsoft.com/en-us/library/ms646989.aspx

typedef	struct	_VS_STRINGFILEINFO
{
	WORD	wSize;			// size of VS_STRINGFILEINFO, including all
					//  subordinate VS_STRINGTABLEs and their VS_STRINGs
	WORD	wCount;			// = 0 (number of characters in string value,
					//      including the terminating NUL character)
	WORD	wType;			// = VS_TEXT
	WCHAR	szKey[15];		// = L"StringFileInfo"
					// (alignment to DWORD not necessary here)
//	WCHAR	szValue[0];		// NO value!
} VS_STRINGFILEINFO;

// https://msdn.microsoft.com/en-us/library/ms646992.aspx

typedef	struct	_VS_STRINGTABLE
{
	WORD	wSize;			// size of VS_STRINGTABLE and all subordinate VS_STRINGs
	WORD	wCount;			// = 0 (number of characters in string value,
					//      including the terminating NUL character)
	WORD	wType;			// = VS_TEXT
	WCHAR	szKey[9];		// = L"040904B0"
					// (alignment to DWORD not necessary here)
//	WCHAR	szValue[0];		// NO value!
} VS_STRINGTABLE;

// https://msdn.microsoft.com/en-us/library/ms646987.aspx

typedef	struct	_VS_STRING
{
	WORD	wSize;			// size of VS_STRING
	WORD	wCount;			// = sizeof(szValue) / sizeof(*szValue)
					//   (number of characters in string value,
					//    including the terminating NUL character)
	WORD	wType;			// = VS_TEXT
	WCHAR	szKey[9];		// = L"Comments"
					//   (mandatory key names are L"CompanyName",
					//    L"FileDescription", L"FileVersion",
					//    L"InternalName", L"OriginalFilename",
					//    L"ProductName", L"ProductVersion";
					//    optional key names are L"Comments",
					//    L"LegalCopyright", L"LegalTrademarks",
					//    L"OleSelfRegister", L"PrivateBuild",
					//    L"SpecialBuild")
					// (alignment to DWORD not necessary here)
	WCHAR	szValue[ANYSIZE_ARRAY];	// = L"..."
	WORD	wPadding;		// = 0 (alignment to DWORD)
} VS_STRING;

// https://msdn.microsoft.com/en-us/library/ms646995.aspx

typedef	struct	_VS_VARFILEINFO
{
	WORD	wSize;			// size of VS_VARFILEINFO and subordinate VS_VAR
	WORD	wCount;			// = 0 (number of characters in string value,
					//      including the terminating NUL character)
	WORD	wType;			// = VS_TEXT
	WCHAR	szKey[12];		// = L"VarFileInfo"
	WORD	wPadding;		// = 0 (alignment to DWORD)
//	WCHAR	szValue[0];		// NO value!
} VS_VARFILEINFO;

// https://msdn.microsoft.com/en-us/library/ms646994.aspx

typedef	struct	_VS_VAR
{
	WORD	wSize;			// = sizeof(VS_VAR)
	WORD	wCount;			// = sizeof(dwValue)
					//   (number of bytes in binary value)
	WORD	wType;			// = VS_BINARY
	WCHAR	szKey[12];		// = L"Translation"
	WORD	wPadding;		// = 0 (alignment to DWORD)
	DWORD	dwValue[ANYSIZE_ARRAY];	// = {0x04B00409, ...}
					//   (pairs of codepage and language ids)
} VS_VAR;

#pragma pack(pop)
#endif // __VERSION_H__
